/* -*- Mode: C; c-basic-offset:8 ; indent-tabs-mode:t -*- */
/*
 * Dynamic library wrapper hiding library symbols from libusb-1.O
 * Copyright (C) 2015 Stanislav Brabec <sbrabec@suse.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "config.h"

/* Include libusb.h with wrapped inlined calls and declarations in style
 * needed for dlopen references */
#include "libusb-wrapped.h"

/* Now make defines to force use of these wrappers */
#define libusb_bulk_transfer (dl_libusb_bulk_transfer)
#define libusb_claim_interface (dl_libusb_claim_interface)
#define libusb_release_interface (dl_libusb_release_interface)
#define libusb_clear_halt (dl_libusb_clear_halt)
#define libusb_reset_device (dl_libusb_reset_device)
#define libusb_get_bus_number (dl_libusb_get_bus_number)
#define libusb_open (dl_libusb_open)
#define libusb_close (dl_libusb_close)
#define libusb_set_configuration (dl_libusb_set_configuration)
#define libusb_control_transfer (dl_libusb_control_transfer)
#define libusb_detach_kernel_driver (dl_libusb_detach_kernel_driver)
#define libusb_exit (dl_libusb_exit)
#define libusb_set_debug (dl_libusb_set_debug)
#define libusb_free_config_descriptor (dl_libusb_free_config_descriptor)
#define libusb_free_device_list (dl_libusb_free_device_list)
#define libusb_ref_device (dl_libusb_ref_device)
#define libusb_unref_device (dl_libusb_unref_device)
#define libusb_get_config_descriptor (dl_libusb_get_config_descriptor)
#define libusb_free_config_descriptor (dl_libusb_free_config_descriptor)
#define libusb_get_device_address (dl_libusb_get_device_address)
#define libusb_get_device_descriptor (dl_libusb_get_device_descriptor)
#define libusb_get_device_list (dl_libusb_get_device_list)
#define libusb_free_device_list (dl_libusb_free_device_list)
#define libusb_get_string_descriptor_ascii (dl_libusb_get_string_descriptor_ascii)
#define libusb_init (dl_libusb_init)
#define libusb_exit (dl_libusb_exit)
#define libusb_set_debug (dl_libusb_set_debug)
#define libusb_interrupt_transfer (dl_libusb_interrupt_transfer)
#define libusb_kernel_driver_active (dl_libusb_kernel_driver_active)
#define libusb_detach_kernel_driver (dl_libusb_detach_kernel_driver)
#define libusb_ref_device (dl_libusb_ref_device)
#define libusb_unref_device (dl_libusb_unref_device)
#define libusb_release_interface (dl_libusb_release_interface)
#define libusb_reset_device (dl_libusb_reset_device)
#define libusb_set_configuration (dl_libusb_set_configuration)
#define libusb_claim_interface (dl_libusb_claim_interface)
#define libusb_set_debug (dl_libusb_set_debug)
#define libusb_set_interface_alt_setting (dl_libusb_set_interface_alt_setting)
#define libusb_clear_halt (dl_libusb_clear_halt)
#define libusb_unref_device (dl_libusb_unref_device)

/* And finally handle the wrapping itself */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <dlfcn.h>

static void *libusb_dl_handle;

#define libusb_dl_set_call(call)\
	if (!(dl_##call = dlsym(libusb_dl_handle, #call)))\
		goto failure;
static inline void libusb_dl_init(void) {
	if (!(libusb_dl_handle = dlopen(LIBUSB_1_0_SONAME, RTLD_NOW|RTLD_LOCAL)))
		goto failure;
	libusb_dl_set_call(libusb_bulk_transfer);
	libusb_dl_set_call(libusb_claim_interface);
	libusb_dl_set_call(libusb_clear_halt);
	libusb_dl_set_call(libusb_get_bus_number);
	libusb_dl_set_call(libusb_open);
	libusb_dl_set_call(libusb_close);
	libusb_dl_set_call(libusb_control_transfer);
	libusb_dl_set_call(libusb_detach_kernel_driver);
	libusb_dl_set_call(libusb_exit);
	libusb_dl_set_call(libusb_free_config_descriptor);
	libusb_dl_set_call(libusb_free_device_list);
	libusb_dl_set_call(libusb_get_config_descriptor);
	libusb_dl_set_call(libusb_get_device_address);
	libusb_dl_set_call(libusb_get_device_descriptor);
	libusb_dl_set_call(libusb_get_device_list);
	libusb_dl_set_call(libusb_get_string_descriptor_ascii);
	libusb_dl_set_call(libusb_init);
	libusb_dl_set_call(libusb_interrupt_transfer);
	libusb_dl_set_call(libusb_kernel_driver_active);
	libusb_dl_set_call(libusb_ref_device);
	libusb_dl_set_call(libusb_release_interface);
	libusb_dl_set_call(libusb_reset_device);
	libusb_dl_set_call(libusb_set_configuration);
	libusb_dl_set_call(libusb_set_debug);
	libusb_dl_set_call(libusb_set_interface_alt_setting);
	libusb_dl_set_call(libusb_unref_device);
	return;
failure:
#ifdef HAVE_GNU_ERRNO_H
	fprintf(stderr, "%s: error while loading " LIBUSB_1_0_SONAME " from libusb-0.1.so.4: %s\n",
		program_invocation_name, dlerror());
#else
	fprintf(stderr, "libusb-compat: error while loading " LIBUSB_1_0_SONAME " from libusb-0.1.so.4: %s\n",
		dlerror());
#endif
	exit(127);
};

static inline void libusb_dl_exit(void) {
	dlclose(libusb_dl_handle);
};
